/*
 *  Copyright 2011 Candlelight Computing.
 *
 *  Candlelight Computing hereby grants this software, in part or
 *  in whole, available under the Apachev2 license.
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  We want to hear from you!  Please let us know how you're using
 *  our products, what you might like to see added (or removed),
 *  or just stop by to say hi!
 *  http://www.candlelightcomputing.com/opensource
 *
 */

package com.candlelightcomputing.test;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.lang.management.ManagementFactory;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Singleton;
import javax.ejb.LocalBean;
import javax.ejb.SessionContext;
import javax.ejb.Startup;
import javax.ejb.Timeout;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;

/**
 *
 * @author steven.siebert
 */
@Singleton
@LocalBean
@Startup
public class TestTimer {

    @Resource
    private SessionContext context;
    private TestConfig config;

    @PostConstruct
    private void init () {
        try {
            Logger.getLogger(TestTimer.class.getName()).log(Level.INFO, "TestTimer initializing...");
            //register MBean
            config = new TestConfig(context.getBusinessObject(TestTimer.class));
            MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
            mbs.registerMBean(config, new ObjectName("test", "timer", "config"));
        } catch (Exception ex) {
            Logger.getLogger(TestTimer.class.getName()).log(Level.SEVERE, null, ex);
        }

        //programmatically setup timer
        setupNextRun();
    }

    @Timeout
    private void doScheduledRun() {
        doRun();
        setupNextRun();
    }

    public void doRun () {
        try {
            Logger.getLogger(TestTimer.class.getName()).log(Level.INFO, "TestTimer has run operation...");
            JAXBContext jc = JAXBContext.newInstance(TestJaxbEntity.class);
            Unmarshaller um = jc.createUnmarshaller();
            TestJaxbEntity e = um.unmarshal(XMLInputFactory.newInstance().createXMLStreamReader(new FileReader("testXml.xml")), TestJaxbEntity.class).getValue();
            System.out.println(e.getOne()+" "+e.getTwo());
        } catch (Exception ex) {
            Logger.getLogger(TestTimer.class.getName()).log(Level.SEVERE, null, ex);
        }
    }


    private void setupNextRun() {
        context.getTimerService().createTimer(config.getFrequency(), null);
    }
}
